package java.security;

import sun.misc.JavaSecurityAccess;
import java.util.Collections;
import java.util.WeakHashMap;
import java.util.Map;
import sun.misc.JavaSecurityProtectionDomainAccess;
import sun.misc.SharedSecrets;
import java.util.Enumeration;
import java.util.ArrayList;
import sun.security.util.SecurityConstants;
import sun.security.util.Debug;

public class ProtectionDomain
{
    private CodeSource codesource;
    private ClassLoader classloader;
    private Principal[] principals;
    private PermissionCollection permissions;
    private boolean hasAllPerm;
    private boolean staticPermissions;
    final ProtectionDomain.Key key;
    private static final Debug debug;
    
    public ProtectionDomain(CodeSource codesource, PermissionCollection permissions) {
        this.hasAllPerm = false;
        this.key = new ProtectionDomain.Key();
        this.codesource = codesource;
        if (permissions != null) {
            (this.permissions = permissions).setReadOnly();
            if (permissions instanceof Permissions && ((Permissions)permissions).allPermission != null) {
                this.hasAllPerm = true;
            }
        }
        this.classloader = null;
        this.principals = new Principal[0];
        this.staticPermissions = true;
    }
    
    public ProtectionDomain(CodeSource codesource, PermissionCollection permissions, ClassLoader classloader, Principal[] array) {
        this.hasAllPerm = false;
        this.key = new ProtectionDomain.Key();
        this.codesource = codesource;
        if (permissions != null) {
            (this.permissions = permissions).setReadOnly();
            if (permissions instanceof Permissions && ((Permissions)permissions).allPermission != null) {
                this.hasAllPerm = true;
            }
        }
        this.classloader = classloader;
        this.principals = ((array != null) ? ((Principal[])array.clone()) : new Principal[0]);
        this.staticPermissions = false;
    }
    
    public final CodeSource getCodeSource() {
        return this.codesource;
    }
    
    public final ClassLoader getClassLoader() {
        return this.classloader;
    }
    
    public final Principal[] getPrincipals() {
        return (Principal[])this.principals.clone();
    }
    
    public final PermissionCollection getPermissions() {
        return this.permissions;
    }
    
    public boolean implies(Permission permission) {
        return this.hasAllPerm || (!this.staticPermissions && Policy.getPolicyNoCheck().implies(this, permission)) || (this.permissions != null && this.permissions.implies(permission));
    }
    
    boolean impliesCreateAccessControlContext() {
        return this.implies(SecurityConstants.CREATE_ACC_PERMISSION);
    }
    
    @Override
    public String toString() {
        String string = "<no principals>";
        if (this.principals != null && this.principals.length > 0) {
            StringBuilder sb = new StringBuilder("(principals ");
            for (int i = 0; i < this.principals.length; ++i) {
                sb.append(this.principals[i].getClass().getName() + " \"" + this.principals[i].getName() + "\"");
                if (i < this.principals.length - 1) {
                    sb.append(",\n");
                }
                else {
                    sb.append(")\n");
                }
            }
            string = sb.toString();
        }
        return "ProtectionDomain  " + this.codesource + "\n " + this.classloader + "\n " + string + "\n " + ((Policy.isSet() && seeAllp()) ? this.mergePermissions() : this.getPermissions()) + "\n";
    }
    
    private static boolean seeAllp() {
        SecurityManager securityManager = System.getSecurityManager();
        if (securityManager == null) {
            return true;
        }
        if (ProtectionDomain.debug != null) {
            if (securityManager.getClass().getClassLoader() == null && Policy.getPolicyNoCheck().getClass().getClassLoader() == null) {
                return true;
            }
        }
        else {
            try {
                securityManager.checkPermission(SecurityConstants.GET_POLICY_PERMISSION);
                return true;
            }
            catch (SecurityException ex) {}
        }
        return false;
    }
    
    private PermissionCollection mergePermissions() {
        if (this.staticPermissions) {
            return this.permissions;
        }
        PermissionCollection collection = (PermissionCollection)AccessController.doPrivileged(new PrivilegedAction<PermissionCollection>() {
            @Override
            public PermissionCollection run() {
                return Policy.getPolicyNoCheck().getPermissions(ProtectionDomain.this);
            }
        });
        Permissions permissions = new Permissions();
        int initialCapacity = 32;
        int initialCapacity2 = 8;
        ArrayList<Object> list = new ArrayList(initialCapacity2);
        ArrayList<Object> list2 = new ArrayList(initialCapacity);
        if (this.permissions != null) {
            synchronized (this.permissions) {
                Enumeration<Permission> elements = this.permissions.elements();
                while (elements.hasMoreElements()) {
                    list.add(elements.nextElement());
                }
            }
        }
        if (collection != null) {
            synchronized (collection) {
                for (Enumeration elements2 = collection.elements(); elements2.hasMoreElements(); ++initialCapacity2) {
                    list2.add(elements2.nextElement());
                }
            }
        }
        if (collection != null && this.permissions != null) {
            synchronized (this.permissions) {
                Enumeration<Permission> elements3 = this.permissions.elements();
                while (elements3.hasMoreElements()) {
                    Permission permission = (Permission)elements3.nextElement();
                    Class<Object> class1 = permission.getClass();
                    String actions = permission.getActions();
                    String name = permission.getName();
                    int i = 0;
                    while (i < list2.size()) {
                        Permission permission2 = (Permission)list2.get(i);
                        if (class1.isInstance(permission2) && name.equals(permission2.getName()) && actions.equals(permission2.getActions())) {
                            list2.remove(i);
                            break;
                        }
                        ++i;
                    }
                }
            }
        }
        if (collection != null) {
            for (int j = list2.size() - 1; j >= 0; --j) {
                permissions.add((Permission)list2.get(j));
            }
        }
        if (this.permissions != null) {
            for (int k = list.size() - 1; k >= 0; --k) {
                permissions.add((Permission)list.get(k));
            }
            return permissions;
        }
        return permissions;
    }
    
    static {
        SharedSecrets.setJavaSecurityAccess(new ProtectionDomain.JavaSecurityAccessImpl());
        debug = Debug.getInstance("domain");
        SharedSecrets.setJavaSecurityProtectionDomainAccess(new JavaSecurityProtectionDomainAccess() {
            public JavaSecurityProtectionDomainAccess.ProtectionDomainCache getProtectionDomainCache() {
                return new JavaSecurityProtectionDomainAccess.ProtectionDomainCache() {
                    private final Map<ProtectionDomain.Key, PermissionCollection> map = Collections.synchronizedMap(new WeakHashMap());
                    
                    public void put(ProtectionDomain protectionDomain, PermissionCollection collection) {
                        this.map.put((protectionDomain == null) ? null : protectionDomain.key, collection);
                    }
                    
                    public PermissionCollection get(ProtectionDomain protectionDomain) {
                        return (protectionDomain == null) ? ((PermissionCollection)this.map.get(null)) : ((PermissionCollection)this.map.get(protectionDomain.key));
                    }
                };
            }
            
            public boolean getStaticPermissionsField(ProtectionDomain protectionDomain) {
                return protectionDomain.staticPermissions;
            }
        });
    }
    
    private static class JavaSecurityAccessImpl implements JavaSecurityAccess
    {
        public <T> T doIntersectionPrivilege(PrivilegedAction<T> privilegedAction, AccessControlContext accessControlContext, AccessControlContext accessControlContext2) {
            if (privilegedAction == null) {
                throw new NullPointerException();
            }
            return AccessController.doPrivileged(privilegedAction, getCombinedACC(accessControlContext2, accessControlContext));
        }
        
        public <T> T doIntersectionPrivilege(PrivilegedAction<T> privilegedAction, AccessControlContext accessControlContext) {
            return this.doIntersectionPrivilege(privilegedAction, AccessController.getContext(), accessControlContext);
        }
        
        private static AccessControlContext getCombinedACC(AccessControlContext accessControlContext, AccessControlContext accessControlContext2) {
            return new AccessControlContext(accessControlContext2.getContext(), new AccessControlContext(accessControlContext, accessControlContext2.getCombiner(), true)).optimize();
        }
    }
    
    final class Key
    {
    }
}
